/*
 * $Id: rb_esmart_container.c 57 2004-08-10 13:33:49Z tilman $
 *
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Esmart/Esmart_Container.h>
#include <evas/rb_evas.h>
#include <evas/rb_evas_object.h>

#include "../rb_esmart.h"

static void c_free (RbEvasObject *e)
{
	c_evas_object_free (e, true);
}

static VALUE c_new (VALUE klass, VALUE evas)
{
	VALUE self, argv[1];
	RbEvasObject *cont;

	CHECK_CLASS (evas, cEvas);
	GET_OBJ (evas, RbEvas, e);

	self = Data_Make_Struct (klass, RbEvasObject, c_evas_object_mark,
	                         c_free, cont);
	cont->real = esmart_container_new (e->real);

	argv[0] = evas;
	rb_obj_call_init (self, 1, argv);

	return self;
}

static VALUE c_append_element (VALUE self, VALUE element)
{
	GET_OBJ (self, RbEvasObject, e);

	CHECK_CLASS (element, cEvasObject);
	GET_OBJ (element, RbEvasObject, o);

	esmart_container_element_append (e->real, o->real);

	return Qnil;
}

static VALUE c_prepend_element (VALUE self, VALUE element)
{
	GET_OBJ (self, RbEvasObject, e);

	CHECK_CLASS (element, cEvasObject);
	GET_OBJ (element, RbEvasObject, o);

	esmart_container_element_prepend (e->real, o->real);

	return Qnil;
}

static VALUE c_remove_element (VALUE self, VALUE element)
{
	GET_OBJ (self, RbEvasObject, e);

	CHECK_CLASS (element, cEvasObject);
	GET_OBJ (element, RbEvasObject, o);

	esmart_container_element_remove (e->real, o->real);

	return Qnil;
}

static VALUE c_direction_get (VALUE self)
{
	GET_OBJ (self, RbEvasObject, e);

	return INT2FIX (esmart_container_direction_get (e->real));
}

static VALUE c_direction_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbEvasObject, e);

	Check_Type (val, T_FIXNUM);

	esmart_container_direction_set (e->real, FIX2INT (val));

	return Qnil;
}

static VALUE c_spacing_get (VALUE self)
{
	GET_OBJ (self, RbEvasObject, e);

	return INT2FIX (esmart_container_spacing_get (e->real));
}

static VALUE c_spacing_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbEvasObject, e);

	Check_Type (val, T_FIXNUM);

	esmart_container_spacing_set (e->real, FIX2INT (val));

	return Qnil;
}

static VALUE c_fill_policy_get (VALUE self)
{
	GET_OBJ (self, RbEvasObject, e);

	return INT2FIX (esmart_container_fill_policy_get (e->real));
}

static VALUE c_fill_policy_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbEvasObject, e);

	Check_Type (val, T_FIXNUM);

	esmart_container_fill_policy_set (e->real, FIX2INT (val));

	return Qnil;
}

static VALUE c_get_padding (VALUE self)
{
	double l = 0, r = 0, t = 0, b = 0;

	GET_OBJ (self, RbEvasObject, e);

	esmart_container_padding_get (e->real, &l, &r, &t, &b);

	return rb_ary_new3 (4, rb_float_new (l), rb_float_new (r),
	                    rb_float_new (t), rb_float_new (b));
}

static VALUE c_set_padding (VALUE self, VALUE l, VALUE r,
                            VALUE t, VALUE b)
{
	GET_OBJ (self, RbEvasObject, e);

	Check_Type (l, T_FLOAT);
	Check_Type (r, T_FLOAT);
	Check_Type (t, T_FLOAT);
	Check_Type (b, T_FLOAT);

	esmart_container_padding_set (e->real, NUM2DBL (l), NUM2DBL (r),
	                              NUM2DBL (t), NUM2DBL (b));

	return Qnil;
}

static VALUE c_scroll (VALUE self, VALUE val)
{
	GET_OBJ (self, RbEvasObject, e);

	Check_Type (val, T_FIXNUM);

	esmart_container_scroll (e->real, FIX2INT (val));

	return Qnil;
}

void Init_esmart_container (void)
{
	VALUE c;

	rb_require ("esmart");

	c = rb_define_class_under (mEsmart, "Container", cEvasObject);

	rb_define_singleton_method (c, "new", c_new, 1);
	rb_define_method (c, "direction", c_direction_get, 0);
	rb_define_method (c, "direction=", c_direction_set, 1);
	rb_define_method (c, "spacing", c_spacing_get, 0);
	rb_define_method (c, "spacing=", c_spacing_set, 1);
	rb_define_method (c, "fill_policy", c_fill_policy_get, 0);
	rb_define_method (c, "fill_policy=", c_fill_policy_set, 1);
	rb_define_method (c, "get_padding", c_get_padding, 0);
	rb_define_method (c, "set_padding", c_set_padding, 4);
	rb_define_method (c, "append_element", c_append_element, 1);
	rb_define_method (c, "prepend_element", c_prepend_element, 1);
	rb_define_method (c, "remove_element", c_remove_element, 1);
	rb_define_method (c, "scroll", c_scroll, 1);

	rb_define_const (c, "HORIZONTAL",
	                 INT2FIX (CONTAINER_DIRECTION_HORIZONTAL));
	rb_define_const (c, "VERTICAL",
	                 INT2FIX (CONTAINER_DIRECTION_VERTICAL));

	rb_define_const (c, "NONE",
	                 INT2FIX (CONTAINER_FILL_POLICY_NONE));
	rb_define_const (c, "KEEP_ASPECT",
	                 INT2FIX (CONTAINER_FILL_POLICY_KEEP_ASPECT));
	rb_define_const (c, "FILL_X",
	                 INT2FIX (CONTAINER_FILL_POLICY_FILL_X));
	rb_define_const (c, "FILL_Y",
	                 INT2FIX (CONTAINER_FILL_POLICY_FILL_Y));
	rb_define_const (c, "HOMOGENOUS",
	                 INT2FIX (CONTAINER_FILL_POLICY_HOMOGENOUS));
}
